@@ -1,78 +1,73 @@
-# Functions called directly by the app
-
-#' List available studies and their metadata
+#' Get study metadata
 #'
-#' @param libraries The directories to search for installed study packages. If
-#'   left as \code{NULL} (the default), then
-#'   \code{\link[utils]{installed.packages}} will use the result of
-#'   \code{\link{.libPaths}}.
+#' Get the study description, version, maintainer, maintainer email, and any
+#' extra metadata added via the argument \code{studyMeta} of
+#' \code{\link{createStudy}}.
 #'
-#' @return Returns a nested list with one element per installed OmicNavigator
-#'   study package. Each study package entry has the following sublist components:
+#' @inheritParams importStudy
 #'
-#'   \item{name}{(character) Name of the study}
-#'   \item{package}{(list) The fields from \code{DESCRIPTION}}
-#'   \item{results}{(nested list) The testIDs available for each modelID}
-#'   \item{enrichments}{(nested list) The annotationIDs available for each modelID}
-#'   \item{plots}{(nested list) The plotIDs available for each modelID}
+#' @return Returns a list with the following components:
+#'
+#'   \item{description}{(character) Study description}
+#'   \item{version}{(character) Study version}
+#'   \item{maintainer}{(character) Study maintainer}
+#'   \item{maintainerEmail}{(character) Study maintainer email}
+#'   \item{studyMeta}{(list) Additional study metadata added via the argument
+#'                    \code{studyMeta} of \code{\link{createStudy})}}
+#'
+#' @seealso \code{\link{createStudy}}
 #'
 #' @export
-listStudies <- function(libraries = NULL) {
-  studies <- getInstalledStudies(libraries = libraries)
-
-  output <- vector(mode = "list", length = length(studies))
-  for (i in seq_along(studies)) {
-    output[[i]] <- list()
-    studyName <- studies[i]
-    output[[i]][["name"]] <- studyName
-
-    # package metadata
-    pkgName <- studyToPkg(studyName)
-    pkgDescription <- utils::packageDescription(pkgName, lib.loc = libraries)
-    pkgDescription <- unclass(pkgDescription)
-    attr(pkgDescription, "file") <- NULL
-    output[[i]][["package"]] <- pkgDescription
-    # For temporary backwards compatibility. The app currently reads
-    # "description"
-    output[[i]][["package"]] <- c(
-      output[[i]][["package"]],
-      list(description = pkgDescription[["Description"]])
-    )
-
-    if (as.package_version(pkgDescription[["OmicNavigatorVersion"]]) <
-        as.package_version(minVersionCompatible)) {
-      warning(
-        "OmicNavigator version incompatibility\n",
-        sprintf("Study \"%s\" was created with version %s\n", studyName,
-                pkgDescription[["OmicNavigatorVersion"]]),
-        sprintf("OmicNavigator version %s is currently installed\n",
-                utils::packageVersion("OmicNavigator")),
-        sprintf("It requires study packages to be created with a minimum OmicNavigator version of %s\n",
-                minVersionCompatible),
-        sprintf("Reinstall the study to avoid any potential issues\n"),
-        immediate. = TRUE
-      )
-    }
-
-    studyDirectory <- getDirectory(studyName, libraries)
-    studySummaryFile <- file.path(studyDirectory, "summary.json")
-    if (!file.exists(studySummaryFile)) {
-      warning(sprintf("Unable to import package %s", pkgName),
-              immediate. = TRUE)
-      next
-    }
-
-    studySummary <- readJson(studySummaryFile, simplifyVector = FALSE)
-    output[[i]] <- c(output[[i]], studySummary)
+getStudyMeta <- function(study, libraries = NULL) {
+  pkg <- studyToPkg(study)
+  if (!requireNamespace(pkg, lib.loc = libraries, quietly = TRUE)) {
+    stop("The package ", pkg, " is not installed")
   }
+  # Need to unload package namespace so that it doesn't interfere with find.package()
+  on.exit(unloadNamespace(pkg), add = TRUE)
 
-  return(output)
+  # Import info from DESCRIPTION
+  description <- packageDescription(pkg, lib.loc = libraries)
+  checkMinVersionCompatible(study, description[["OmicNavigatorVersion"]])
+  if (is.null(description[["Maintainer"]])) {
+    description[["Maintainer"]] <- "Unknown <unknown@unknown>"
+    message(
+      "This study package didn't have a maintainer listed",
+      "\nUsing the placeholder: ", description[["Maintainer"]],
+      "\nHighly recommended to update this with your own name and email:",
+      "\n<name of study object>$maintainer <- \"<Your Name>\"",
+      "\n<name of study object>$maintainer <- \"<youremail@domain.com>\"",
+      "\n(replace the text in between the brackets; make sure to delete the brackets)"
+    )
+  }
+  maintainerField <- strsplit(description[["Maintainer"]], "<|>")[[1]]
+  maintainer <- sub("[[:space:]]$", "", maintainerField[1])
+  maintainerEmail <- maintainerField[2]
+  descriptionFieldsReservedFile <- system.file(
+    "extdata/description-fields-reserved.txt",
+    package = "OmicNavigator",
+    mustWork = TRUE
+  )
+  descriptionFieldsReserved <- scan(
+    file = descriptionFieldsReservedFile,
+    what = character(),
+    quiet = TRUE
+  )
+  studyMeta <- description[setdiff(names(description), descriptionFieldsReserved)]
+
+  studyMeta <- c(
+    description = description[["Description"]],
+    version = description[["Version"]],
+    maintainer = maintainer,
+    maintainerEmail = maintainerEmail,
+    studyMeta = list(studyMeta)
+  )
+  return(studyMeta)
 }
 
 #' Get results table from a study
 #'
 #' @inheritParams shared-get
-#' @inheritParams listStudies
 #'
 #' @return A data frame which includes the columns from the features table
 #'   followed by the columns from the results table. All the columns from the
@@ -85,25 +80,34 @@ listStudies <- function(libraries = NULL) {
 #'
 #' @export
 getResultsTable <- function(study, modelID, testID, annotationID = NULL, termID = NULL, libraries = NULL) {
-  results <- getResults(study, modelID, testID)
-  features <- getFeatures(study, modelID, quiet = TRUE)
-
+  results <- getResults(study, modelID, testID, libraries = libraries)
   if (isEmpty(results)) return(data.frame())
-  if (isEmpty(features)) return(results)
 
-  # Results must be first argument to preserve input order
-  resultsTable <- merge(results, features, by = 1,
-                        all.x = TRUE, all.y = FALSE, sort = FALSE)
-  # Rearrange columns so that features are listed first
-  columnsOrder <- c(colnames(features),
-                    setdiff(colnames(results), colnames(features)))
-  resultsTable <- resultsTable[, columnsOrder]
+  # Add feature columns if available
+  features <- getFeatures(study, modelID, quiet = TRUE, libraries = libraries)
+  if (isEmpty(features)) {
+    resultsTable <- results
+  } else {
+    # Results must be first argument to preserve input order
+    resultsTable <- merge(results, features, by = 1,
+                          all.x = TRUE, all.y = FALSE, sort = FALSE)
+    # Rearrange columns so that features are listed first
+    columnsOrder <- c(colnames(features),
+                      setdiff(colnames(results), colnames(features)))
+    resultsTable <- resultsTable[, columnsOrder]
+  }
 
+  # Filter by annotationID/termID if provided
   if (!is.null(annotationID) && !is.null(termID)) {
     termFeatures <- getNodeFeatures(study, annotationID, termID, libraries = libraries)
-    annotationIDfeatureID <- getAnnotations(study, annotationID = annotationID)[["featureID"]]
-    featureIDcolumn <- which(colnames(resultsTable) == annotationIDfeatureID)
-    resultsTable <- resultsTable[resultsTable[[featureIDcolumn]] %in% termFeatures, ]
+    if (isEmpty(termFeatures)) {
+      return(data.frame())
+    } else {
+      annotations <- getAnnotations(study, annotationID = annotationID, libraries = libraries)
+      annotationIDfeatureID <- annotations[["featureID"]]
+      featureIDcolumn <- which(colnames(resultsTable) == annotationIDfeatureID)
+      resultsTable <- resultsTable[resultsTable[[featureIDcolumn]] %in% termFeatures, ]
+    }
   }
 
   return(resultsTable)
@@ -113,7 +117,6 @@ getResultsTable <- function(study, modelID, testID, annotationID = NULL, termID
 #'
 #' @inheritParams shared-get
 #' @inheritParams shared-upset
-#' @inheritParams listStudies
 #'
 #' @return A data frame of enrichments with the following columns:
 #'
@@ -144,7 +147,6 @@ getEnrichmentsTable <- function(study, modelID, annotationID, type = "nominal",
 #' Get enrichments network from a study
 #'
 #' @inheritParams shared-get
-#' @inheritParams listStudies
 #'
 #' @return Returns a list with the following components:
 #'
@@ -154,7 +156,6 @@ getEnrichmentsTable <- function(study, modelID, annotationID, type = "nominal",
 #'   \item{links}{(list) The statistics for each pairwise overlap between the
 #'   annotation terms (i.e. nodes)}
 #'
-#' @importFrom data.table ":=" "%chin%" .N
 #' @export
 getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL) {
 
@@ -170,17 +171,22 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
     message(sprintf("No terms available for annotationID \"%s\"", annotationID))
     return(list())
   }
-  terms <- data.table::data.table(
+  terms <- data.table(
     termID = names(termsVec),
     geneSetSize = lengths(termsVec)
   )
 
-  enrichments <- getEnrichments(study, modelID = modelID, annotationID = annotationID)
+  enrichments <- getEnrichments(
+    study,
+    modelID = modelID,
+    annotationID = annotationID,
+    libraries = libraries
+  )
   if (isEmpty(enrichments)) return(list())
   enrichmentsTable <- combineListIntoTable(enrichments, "testID")
-  data.table::setDT(enrichmentsTable)
+  setDT(enrichmentsTable)
 
-  nodesLong <- data.table::merge.data.table(
+  nodesLong <- merge.data.table(
     x = enrichmentsTable,
     y = terms,
     by = "termID",
@@ -188,7 +194,7 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
     all.y = FALSE,
     sort = FALSE
   )
-  data.table::setorderv(nodesLong, cols = "testID")
+  setorderv(nodesLong, cols = "testID")
 
   tests <- unique(nodesLong[["testID"]])
 
@@ -199,7 +205,7 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
     by = c("termID", "description", "geneSetSize")
   ]
   nodes[, id := seq_len(.N)]
-  data.table::setcolorder(nodes, "id")
+  setcolorder(nodes, "id")
 
   overlaps <- getOverlaps(
     study,
@@ -208,8 +214,8 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
   )
   if (isEmpty(overlaps)) return(list())
 
-  links <- data.table::setDT(overlaps)
-  data.table::setnames(
+  links <- setDT(overlaps)
+  setnames(
     links,
     old = c("term1", "term2"),
     new = c("source", "target")
@@ -217,16 +223,16 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
   links <- links[links[["source"]] %chin% nodes[["termID"]] &
                    links[["target"]] %chin% nodes[["termID"]], ]
   links[, id := seq_len(.N)]
-  data.table::setcolorder(links, "id")
+  setcolorder(links, "id")
 
   # Use node IDs with links
-  links[["source"]] <- data.table::chmatch(links[["source"]], nodes[["termID"]])
-  links[["target"]] <- data.table::chmatch(links[["target"]], nodes[["termID"]])
+  links[["source"]] <- chmatch(links[["source"]], nodes[["termID"]])
+  links[["target"]] <- chmatch(links[["target"]], nodes[["termID"]])
 
   enrichmentsNetwork <- list(
     tests = tests,
-    nodes = data.table::setDF(nodes),
-    links = data.table::setDF(links)
+    nodes = setDF(nodes),
+    links = setDF(links)
   )
 
   return(enrichmentsNetwork)
@@ -237,7 +243,6 @@ getEnrichmentsNetwork <- function(study, modelID, annotationID, libraries = NULL
 #' @param study An OmicNavigator study. Only accepts name of installed study
 #'   package.
 #' @inheritParams shared-get
-#' @inheritParams listStudies
 #'
 #' @return Returns a character vector with the features in the termID
 #'
@@ -282,10 +287,10 @@ getNodeFeatures <- function(study, annotationID, termID, libraries = NULL) {
 #' @seealso \code{\link{getNodeFeatures}}
 #'
 #' @export
-getLinkFeatures <- function(study, annotationID, termID1, termID2) {
+getLinkFeatures <- function(study, annotationID, termID1, termID2, libraries = NULL) {
 
-  nodeFeatures1 <- getNodeFeatures(study, annotationID, termID1)
-  nodeFeatures2 <- getNodeFeatures(study, annotationID, termID2)
+  nodeFeatures1 <- getNodeFeatures(study, annotationID, termID1, libraries = libraries)
+  nodeFeatures2 <- getNodeFeatures(study, annotationID, termID2, libraries = libraries)
 
   linkFeatures <- sort(intersect(nodeFeatures1, nodeFeatures2))
 
@@ -303,8 +308,8 @@ getLinkFeatures <- function(study, annotationID, termID1, termID2) {
 #' @seealso \code{\link{addMetaFeatures}}, \code{\link{getMetaFeatures}}
 #'
 #' @export
-getMetaFeaturesTable <- function(study, modelID, featureID) {
-  metaFeatures <- getMetaFeatures(study, modelID = modelID)
+getMetaFeaturesTable <- function(study, modelID, featureID, libraries = NULL) {
+  metaFeatures <- getMetaFeatures(study, modelID = modelID, libraries = libraries)
   if (isEmpty(metaFeatures)) return(data.frame())
 
   metaFeaturesTable <- metaFeatures[metaFeatures[, 1] == featureID, ]
@@ -338,11 +343,11 @@ getMetaFeaturesTable <- function(study, modelID, featureID) {
 #' @seealso \code{\link{addBarcodes}}, \code{\link{getBarcodes}}
 #'
 #' @export
-getBarcodeData <- function(study, modelID, testID, annotationID, termID) {
+getBarcodeData <- function(study, modelID, testID, annotationID, termID, libraries = NULL) {
 
-  resultsTable <- getResultsTable(study, modelID = modelID, testID = testID)
+  resultsTable <- getResultsTable(study, modelID = modelID, testID = testID, libraries = libraries)
   if (isEmpty(resultsTable)) return(list())
-  barcodes <- getBarcodes(study, modelID = modelID)
+  barcodes <- getBarcodes(study, modelID = modelID, libraries = libraries)
   if (isEmpty(barcodes)) return(list())
 
   # Default barcode settings. See ?addBarcodes
@@ -370,7 +375,7 @@ getBarcodeData <- function(study, modelID, testID, annotationID, termID) {
          barcodes[["statistic"]]))
   }
 
-  annotations <- getAnnotations(study, annotationID = annotationID)
+  annotations <- getAnnotations(study, annotationID = annotationID, libraries = libraries)
   if (isEmpty(annotations)) return(list())
   if (!termID %in% names(annotations[["terms"]])) {
     stop(sprintf("The term \"%s\" is not available for the annotation \"%s\"",
@@ -459,14 +464,14 @@ getBarcodeData <- function(study, modelID, testID, annotationID, termID) {
 #'   modelID, an empty character vector is returned.
 #'
 #' @export
-getReportLink <- function(study, modelID) {
-  report <- getReports(study, modelID = modelID)
+getReportLink <- function(study, modelID, libraries = NULL) {
+  report <- getReports(study, modelID = modelID, libraries = libraries)
   if (isEmpty(report)) return(character())
 
   if (isUrl(report)) return(report)
 
   pkgName <- studyToPkg(study)
-  installationDir <- dirname(find.package(package = pkgName))
+  installationDir <- dirname(find.package(package = pkgName, lib.loc = libraries))
   reportFile <- file.path(installationDir, report)
   if (!file.exists(reportFile)) {
     stop(sprintf("The requested report file does not exist: %s", reportFile))
@@ -481,12 +486,17 @@ getReportLink <- function(study, modelID) {
 #' OmicNavigator package functions via OpenCPU than to call the utils package
 #' for this one endpoint.
 #'
+#' @param libraries Directory path(s) to R package library(ies). Passed to the
+#'   argument \code{lib.loc} of \code{\link[utils]{packageVersion}}.
+#'
 #' @return Returns a one-element character vector with the version of the
 #'   currently installed OmicNavigator R package
 #'
+#' @seealso \code{\link[utils]{packageVersion}}
+#'
 #' @export
-getPackageVersion <- function() {
-  as.character(utils::packageVersion("OmicNavigator"))
+getPackageVersion <- function(libraries = NULL) {
+  as.character(packageVersion("OmicNavigator", lib.loc = libraries))
 }
 
 #' Get favicon URLs for table linkouts
