@@ -1,7 +1,23 @@
 # The minimum version of OmicNavigator that is still compatible with the current
 # version. If a study package is created with a version less than this minimum
-# version, then listStudies() will throw a warning.
-minVersionCompatible <- "0.24.0"
+# version, then getStudyMeta() throws a warning.
+checkMinVersionCompatible <- function(study, studyPackageVersion, minVersionCompatible = "0.24.0") {
+  studyPackageVersion <- as.package_version(studyPackageVersion)
+  minVersionCompatible <- as.package_version(minVersionCompatible)
+  if (studyPackageVersion < minVersionCompatible) {
+    warning(
+      "OmicNavigator version incompatibility\n",
+      sprintf("Study \"%s\" was created with version %s\n",
+              study, studyPackageVersion),
+      sprintf("OmicNavigator version %s is currently installed\n",
+              packageVersion("OmicNavigator")),
+      sprintf("It requires study packages to be created with a minimum OmicNavigator version of %s\n",
+              minVersionCompatible),
+      sprintf("Reinstall the study to avoid any potential issues\n"),
+      immediate. = TRUE
+    )
+  }
+}
 
 # The default prefix added to OmicNavigator study packages. Used by .onLoad()
 # below and by getPrefix() in utility.R. The default is overriden by setting
@@ -10,7 +26,7 @@ OmicNavigatorPrefix <- "ONstudy"
 
 # The R package is meant to be used with a specific version of the app. If a
 # user has an older or newer version installed, send a warning.
-versionAppPinned <- "2.0.0"
+versionAppPinned <- "2.3.0"
 
 # The extra packages required to run the app
 appPackages <- c(
@@ -41,7 +57,7 @@ appPackages <- c(
 
 # Report versions of package and app when package is first attached
 .onAttach <- function(libname, pkgname) {
-  versionPackage <- utils::packageVersion("OmicNavigator")
+  versionPackage <- packageVersion("OmicNavigator")
   versionPackageMessage <- sprintf("OmicNavigator R package version: %s",
                                    versionPackage)
   packageStartupMessage(versionPackageMessage)
@@ -49,7 +65,7 @@ appPackages <- c(
   appDir <- system.file("www", package = "OmicNavigator")
   manifestFile <- file.path(appDir, "manifest.json")
   if (file.exists(manifestFile)) {
-    manifest <- jsonlite::read_json(manifestFile)
+    manifest <- read_json(manifestFile)
     versionApp <- manifest[["version"]]
   } else {
     versionApp <- NA_character_
@@ -78,12 +94,19 @@ appPackages <- c(
 #' options(OmicNavigator.prefix = "OmicNavigatorStudy")
 #' }
 #'
+#' @importFrom data.table ":=" "%chin%" .N as.data.table chmatch data.table
+#'   dcast.data.table fread fwrite melt merge.data.table setcolorder setDF setDT
+#'   setnames setorderv
+#' @importFrom graphics barplot par plot text
+#' @importFrom jsonlite read_json write_json
+#' @importFrom stats median pnorm prcomp rnorm runif setNames
+#' @importFrom tools file_ext
+#' @importFrom utils capture.output download.file getFromNamespace head
+#'   install.packages installed.packages modifyList packageDescription
+#'   packageVersion remove.packages unzip
+#'
 "_PACKAGE"
 
-# Make the internal package functions "data.table aware"
-# https://rdatatable.gitlab.io/data.table/articles/datatable-importing.html#data-table-in-imports-but-nothing-imported-1
-.datatable.aware <- TRUE
-
 # Avoid NOTE from `R CMD check`: no visible binding for global variable '.data'
 .data <- NULL
 # Used by {ggplot2} for example plots in tests.R. The official guidance is to
